import React, { useState, useEffect } from 'react';
import { TableBody, TableCell, TableRow } from '@mui/material';
import {
    StyledTable,
    StyledTableContainer,
    StyledEmptyRow,
    StyledProgressCell,
    StyledLinearProgress,
} from './NsTable.styled';
import type { NsTableProps, Item, OrderType } from './types';
import { stableSort, getComparator } from './helper';
import NsTableHeader from './NsTableHeader';
import NsTablePagination from './NsTablePagination';
import ExpandableRow from './NsExpandableRow';
import Grid from '@mui/material/Grid';

export default function NsTable({
    headers = [],
    items = [],
    rows,
    rowsPerPageOptions,
    itemKey,
    expandableContent,
    actionClick,
    showActions = true,
    showExpandedContent = true,
    loading = false,
    isExpandableAll = false,
}: NsTableProps) {
    const [order, setOrder] = useState<OrderType>('asc');
    const [orderBy, setOrderBy] = useState<keyof Item>(itemKey);
    const [expandMap, setExpandMap] = useState<{ [key: number]: boolean }>({});
    const [page, setPage] = useState(0);
    const [rowsPerPage, setRowsPerPage] = useState(rows);
    const itemKeyList = React.useMemo(() => {
        return items.length ? Object.keys(items[0]) : [];
    }, [items]);

    useEffect(() => {
        setRowsPerPage(rows);
        setPage(0);
    }, [rows]);

    const handleRequestSort = (event: React.MouseEvent<unknown>, property: keyof Item) => {
        const isAsc = orderBy === property && order === 'asc';
        setOrder(isAsc ? 'desc' : 'asc');
        setOrderBy(property);
    };

    const handleClick = (index: number) => {
        if (isExpandableAll) {
            setExpandMap((prevState) => ({ ...prevState, [index]: !prevState[index] }));
        } else {
            setExpandMap({ [index]: !expandMap[index] });
        }
    };

    const handleChangePage = (newPage: number) => {
        setPage(newPage - 1);
    };

    const emptyRows = getEmptyRows();

    const visibleRows = React.useMemo(
        () =>
            stableSort(items, getComparator(order, orderBy)).slice(
                page * rowsPerPage,
                page * rowsPerPage + rowsPerPage
            ),
        [order, orderBy, page, rowsPerPage, items]
    );

    return (
        <div>
            <StyledTableContainer>
                <StyledTable>
                    <NsTableHeader
                        headCells={headers}
                        order={order}
                        orderBy={orderBy as string}
                        onRequestSort={handleRequestSort}
                        rowCount={items.length}
                    />
                    <TableBody>
                        {loading ? (
                            <TableRow>
                                <StyledProgressCell colSpan={itemKeyList.length}>
                                    <Grid container>
                                        <Grid item xs={12}>
                                            <StyledLinearProgress />
                                        </Grid>
                                    </Grid>
                                </StyledProgressCell>
                            </TableRow>
                        ) : null}
                        {visibleRows.map((row, index) => {
                            return (
                                <ExpandableRow
                                    key={index}
                                    expandableContent={expandableContent}
                                    isExpand={expandMap[index]}
                                    handleClick={handleClick}
                                    row={row}
                                    index={index}
                                    itemKeyList={itemKeyList}
                                    actionClick={actionClick}
                                    showActions={showActions}
                                    showExpandedContent={showExpandedContent}
                                />
                            );
                        })}
                        {emptyRows > 0 ? (
                            <StyledEmptyRow emptyRows={emptyRows}>
                                <TableCell colSpan={itemKeyList.length} />
                            </StyledEmptyRow>
                        ) : null}
                    </TableBody>
                </StyledTable>
            </StyledTableContainer>
            <NsTablePagination
                rowsPerPageOptions={rowsPerPageOptions}
                count={items.length}
                rowsPerPage={rowsPerPage}
                page={page + 1}
                onPageChange={handleChangePage}
            />
        </div>
    );

    function getEmptyRows() {
        return page > 0 ? Math.max(0, (1 + page) * rowsPerPage - items.length) : 0;
    }
}
